defmodule Feedparser.Atom do
  alias Feedparser.Feed
  alias Feedparser.Item

  def parse(raw) do
    {_, _, feed, items} = :erlsom.sax(raw, [], fn(e,s) -> handle_event(e,s) end)
    %{feed | items: items}
  end

  # Document / Element

  def handle_event(:startDocument, []) do
    {"", :feed, %Feed{}, []}
  end

  def handle_event({:startElement, _uri, 'entry', _pre, _attr}, {_tag, _context, feed, items}) do
    {"", :item, feed, [%Item{}|items]}
  end

  def handle_event({:endElement, _uri, 'entry', _pre}, {_tag, _context, feed, items}) do
    {"", :feed, feed, items}
  end

  def handle_event({:startElement, _uri, 'link', _pre, [{:attribute, 'href',[],[],href}]}, {_tag, :feed, feed, items}) do
    {"", :feed, %{feed | link: href}, items}
  end

  def handle_event({:startElement, _uri, name, _pre, _attr}, {_tag, context, feed, items}) do
    {name, context, feed, items}
  end

  # Feed

  # Note: 'link' is handled in line 24
  feed_events = [{'title', :title}, {'subtitle', :description},
                 {'updated', :lastBuildDate}]

  for {from, to} <- feed_events do
    def handle_event({:characters, c}, {unquote(from), :feed, feed, items}) do
      {"", :feed, %{feed | unquote(to) => c}, items}
    end
  end

  # Item

  item_events = [{'title', :title}, {'link', :link}, {'summary', :description},
                 {'updated', :pubDate}]

  for {from, to} <- item_events do
    def handle_event({:characters, c}, {unquote(from), :item, feed, [item|t]}) do
      {"", :item, feed, [%{item | unquote(to) => c} | t]}
    end
  end

  def handle_event(:endDocument, state), do: state
  def handle_event(_, state), do: state
end
