defmodule Feedparser.RSS do
  alias Feedparser.Feed
  alias Feedparser.Item

  def parse(raw) do
    {_, _, feed, items} = :erlsom.sax(raw, [], fn(e,s) -> handle_event(e,s) end)
    %{feed | items: items}
  end

  # Document / Element

  def handle_event(:startDocument, []), do: {"", :feed, %Feed{}, []}

  def handle_event({:startElement, _uri, 'item', _pre, _attr}, {_tag, _context, feed, items}) do
    {"", :item, feed, [%Item{}|items]}
  end

  def handle_event({:endElement, _uri, 'item', _pre}, {_tag, _context, feed, items}) do
    {"", :feed, feed, items}
  end

  def handle_event({:startElement, _uri, name, _pre, _attr}, {_tag, context, feed, items}) do
    {name, context, feed, items}
  end

  # Feed

  feed_events = [{'title', :title}, {'link', :link},
                 {'description', :description},
                 {'lastBuildDate', :lastBuildDate}]

  for {from, to} <- feed_events do
    def handle_event({:characters, c}, {unquote(from), :feed, feed, items}) do
      {"", :feed, %{feed | unquote(to) => c}, items}
    end
  end

  # Item

  item_events = [{'title', :title}, {'link', :link},
                 {'description', :description}, {'pubDate', :pubDate}]

  for {from, to} <- item_events do
    def handle_event({:characters, c}, {unquote(from), :item, feed, [item|t]}) do
      {"", :item, feed, [%{item | unquote(to) => c} | t]}
    end
  end

  def handle_event(:endDocument, state), do: state
  def handle_event(_, state), do: state
end
